@@ -22,7 +22,11 @@ from openscm_units import unit_registry as ur
 
 from ._base import OpsMixin
 from ._xarray import inject_xarray_methods
-from .errors import MissingRequiredColumnError, NonUniqueMetadataError
+from .errors import (
+    DuplicateTimesError,
+    MissingRequiredColumnError,
+    NonUniqueMetadataError,
+)
 from .filters import (
     HIERARCHY_SEPARATOR,
     datetime_match,
@@ -478,12 +482,17 @@ class BaseScmRun(OpsMixin):  # pylint: disable=too-many-public-methods
 
             (_df, _meta) = _read_file(data, required_cols=self.required_cols, **kwargs)
 
-        self._time_points = TimePoints(_df.index.values)
+        if _df.index.duplicated().any():
+            raise DuplicateTimesError(_df.index)
+
+        # use :class:`TimePoints` to sort times before continuing
+        _df.index = TimePoints(_df.index.values).to_index()
+        _df = _df.sort_index()
 
         _df = _df.astype(float)
         self._df = _df
-        self._df.index = self._time_points.to_index()
-        self._df = self._df.sort_index()
+        # set time points using the sorted times
+        self._time_points = TimePoints(_df.index.values)
         self._meta = pd.MultiIndex.from_frame(_meta.astype("category"))
 
     def copy(self):
@@ -1266,7 +1275,7 @@ class BaseScmRun(OpsMixin):  # pylint: disable=too-many-public-methods
         extrapolation_type: str = "linear",
     ):
         """
-        Interpolate the dataframe onto a new time frame.
+        Interpolate the data onto a new time frame.
 
         Parameters
         ----------
@@ -1337,7 +1346,7 @@ class BaseScmRun(OpsMixin):  # pylint: disable=too-many-public-methods
 
         Examples
         --------
-        Resample a dataframe to annual values
+        Resample a run to annual values
 
         >>> scm_df = ScmRun(
         ...     pd.Series([1, 2, 10], index=(2000, 2001, 2009)),
@@ -1537,20 +1546,21 @@ class BaseScmRun(OpsMixin):  # pylint: disable=too-many-public-methods
 
         na_override: [int, float]
             Convert any nan value in the timeseries meta to this value during processsing.
-            The meta values converted back to nan's before the dataframe is returned. This
+            The meta values converted back to nan's before the run is returned. This
             should not need to be changed unless the existing metadata clashes with the
             default na_override value.
 
-            This functionality is disabled if na_override is None, but may result incorrect
+            This functionality is disabled if na_override is None, but may result in incorrect
             results if the timeseries meta includes any nan's.
 
         **kwargs
-            Keyword arguments to pass to the pandas operation
+            Keyword arguments to pass ``operation`` (or the pandas operation if ``operation``
+            is a string)
 
         Returns
         -------
         :class:`pandas.DataFrame`
-            The quantiles of the timeseries, grouped by all columns in :attr:`meta`
+            The result of ``operation``, grouped by all columns in :attr:`meta`
             other than :obj:`cols`
 
         Raises
@@ -1569,6 +1579,7 @@ class BaseScmRun(OpsMixin):  # pylint: disable=too-many-public-methods
                     "na_override clashes with existing meta: {}".format(na_override)
                 )
             ts.index = pd.MultiIndex.from_frame(ts_idx.fillna(na_override))
+
         group_cols = list(set(ts.index.names) - set(cols))
         grouper = ts.groupby(group_cols)
 
@@ -1669,6 +1680,13 @@ class BaseScmRun(OpsMixin):  # pylint: disable=too-many-public-methods
 
         return out
 
+    @staticmethod
+    def _check_groupby_input(v):
+        if len(v) == 1 and not isinstance(v[0], str):
+            v = tuple(v[0])
+
+        return v
+
     def groupby(self, *group):
         """
         Group the object by unique metadata
@@ -1705,10 +1723,49 @@ class BaseScmRun(OpsMixin):  # pylint: disable=too-many-public-methods
         -------
         :class:`RunGroupBy`
             See the documentation for :class:`RunGroupBy` for more information
-
         """
-        if len(group) == 1 and not isinstance(group[0], str):
-            group = tuple(group[0])
+        group = self._check_groupby_input(group)
+
+        return RunGroupBy(self, group)
+
+    def get_meta_columns_except(self, *not_group):
+        """
+        Get columns in meta except a set
+
+        Parameters
+        ----------
+        not_group: str or list of str
+            Columns to exclude from the grouping
+
+        Returns
+        -------
+        list
+            Meta columns except the ones supplied (sorted alphabetically)
+        """
+        not_group = self._check_groupby_input(not_group)
+        group = sorted(tuple(set(self.meta.columns) - set(not_group)))
+
+        return group
+
+    def groupby_all_except(self, *not_group):
+        """
+        Group the object by unique metadata apart from the input columns
+
+        In other words, the groups are determined by all columns in
+        ``self.meta`` except for  those in ``not_group``
+
+        Parameters
+        ----------
+        not_group: str or list of str
+            Columns to exclude from the grouping
+
+        Returns
+        -------
+        :class:`RunGroupBy`
+            See the documentation for :class:`RunGroupBy` for more information
+        """
+        group = self.get_meta_columns_except(not_group)
+
         return RunGroupBy(self, group)
 
     def convert_unit(
@@ -1875,7 +1932,7 @@ class BaseScmRun(OpsMixin):  # pylint: disable=too-many-public-methods
         **kwargs: Any,
     ):
         """
-        Append additional data to the current dataframe.
+        Append additional data to the current data.
 
         For details, see :func:`run_append`.
 
@@ -1927,6 +1984,49 @@ class BaseScmRun(OpsMixin):  # pylint: disable=too-many-public-methods
             metadata=metadata,
         )
 
+    def append_timewise(
+        self, other, align_columns,
+    ):
+        """
+        Append timeseries along the time axis
+
+        Parameters
+        ----------
+        other : :obj:`scmdata.ScmRun`
+            :obj:`scmdata.ScmRun` containing the timeseries to append
+
+        align_columns : list
+            Columns used to align ``other`` and ``self`` when joining
+
+        Returns
+        -------
+        :obj:`scmdata.ScmRun`
+            Result of joining ``self`` and ``other`` along the time axis
+        """
+        ts_self = self.timeseries()
+        try:
+            ts_other = other.timeseries(meta=align_columns)
+        except NonUniqueMetadataError as exc:
+            error_msg = (
+                "Calling ``other.timeseries(meta=align_columns)`` must "
+                "result in umabiguous timeseries"
+            )
+            raise ValueError(error_msg) from exc
+
+        ts_other_aligned, ts_self_aligned = ts_other.align(ts_self)
+        ts_self_aligned = ts_self_aligned.dropna(how="all", axis="columns")
+        ts_other_aligned = ts_other_aligned.dropna(how="all", axis="columns")
+
+        # if ts_other_aligned.isnull().any(axis=1):
+        #     warning?
+
+        out = pd.concat([ts_other_aligned, ts_self_aligned], axis=1)
+
+        try:
+            return type(self)(out)
+        except DuplicateTimesError as exc:
+            raise ValueError("``self`` and ``other`` have overlapping times") from exc
+
     def to_iamdataframe(self) -> LongDatetimeIamDataFrame:  # pragma: no cover
         """
         Convert to a :class:`LongDatetimeIamDataFrame` instance.
